# -*- coding: utf-8 -*-

import grok
from unicodedata import normalize
from dolmen.content import IBaseContent, IContainer
from zope.container.interfaces import INameChooser

ATTEMPTS = 100


class NormalizingNameChooser(grok.Adapter):
    grok.context(IContainer)
    grok.implements(INameChooser)

    def checkName(self, name, object):
        return not name in self.context

    def _findUniqueName(self, name, object):

        if not name in self.context:
            return name

        idx = 1
        while idx <= ATTEMPTS:
            new_name = "%s_%d" % (name, idx)
            if not new_name in self.context:
                return new_name
            idx += 1

        raise ValueError(
            "Cannot find a unique name based on "
            "`%s` after %d attemps." % (name, ATTEMPTS))

    def chooseName(self, name, object):
        if not name:
            if IBaseContent.providedBy(object):
                name = object.title.strip()
                ascii = normalize('NFKD', name).encode('ascii', 'ignore')
                name = ascii.replace(' ', '_').lower()
            else:
                NotImplementedError(
                    """NormalizingNameChooser can't choose a name if the
                    parameter name is omited and if the component has no
                    adapter to INameFromTitle""")

        return self._findUniqueName(name, object)



# -*- coding: utf-8 -*-

import doctest
import unittest
import dolmen.app.container
import zope.component
from zope.component.interfaces import IComponentLookup
from zope.configuration import xmlconfig
from zope.container.interfaces import ISimpleReadContainer
from zope.container.traversal import ContainerTraversable
from zope.interface import Interface
from zope.security.management import newInteraction, endInteraction
from zope.security.testing import Principal, Participation
from zope.site.folder import rootFolder
from zope.site.hooks import getSite
from zope.site.site import LocalSiteManager, SiteManagerAdapter
from zope.testing import module
from zope.traversing.interfaces import ITraversable
from zope.traversing.testing import setUp


def SiteSetUp(test):
    module.setUp(test, 'dolmen.app.container.ftests')

    zope.component.hooks.setHooks()

    # Set up site manager adapter
    zope.component.provideAdapter(
        SiteManagerAdapter, (Interface,), IComponentLookup)

    # Set up traversal
    setUp()
    zope.component.provideAdapter(
        ContainerTraversable, (ISimpleReadContainer,), ITraversable)

    xmlconfig.file('ftesting.zcml', dolmen.app.container)

    # Set up site
    site = rootFolder()
    site.setSiteManager(LocalSiteManager(site))
    zope.component.hooks.setSite(site)

    # Creating a simple interaction with "manager"
    participation = Participation(Principal('zope.mgr'))
    newInteraction(participation)


def tearDown(test):
    module.tearDown(test)
    endInteraction()
    zope.component.hooks.resetHooks()
    zope.component.hooks.setSite()


def test_suite():
    suite = unittest.TestSuite()
    readme = doctest.DocFileSuite(
        'README.txt', setUp=SiteSetUp, tearDown=tearDown,
        globs={"getSite": getSite},
        optionflags=(doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE),
        )
    #readme.layer = FunctionalLayer
    suite.addTest(readme)
    return suite



from zope.i18nmessageid import MessageFactory
MF = MessageFactory("dolmen.app.container")

from dolmen.app.container.addtofolder import AddMenu
from dolmen.app.container.namechoosers import NormalizingNameChooser
from dolmen.app.container.listing import FolderListing



# -*- coding: utf-8 -*-

import grok

from dolmen import menu
from dolmen.app import security, layout
from dolmen.app.container import MF as _
from megrok.z3ctable import LinkColumn, ModifiedColumn, table

from ZODB.broken import PersistentBroken
from zope.container.interfaces import IContainer
from zope.i18n import translate
from zope.interface import Interface

grok.templatedir("templates")


@menu.menuentry(layout.ContextualMenu, order=40)
class FolderListing(layout.TablePage):
    grok.title(_(u"Content"))
    grok.context(IContainer)
    grok.require(security.CanListContent)

    batchSize = 20
    startBatchingAt = 20
    cssClassEven = u'even'
    cssClassOdd = u'odd'
    cssClasses = {'table': 'listing sortable'}
    sortOn = None
    label = _("Content of the folder")

    @property
    def values(self):
        return self.context.values()

    def update(self):
        layout.TablePage.update(self)
        self.table = self.renderTable()
        self.batch = self.renderBatch()


class Title(LinkColumn):
    """Displays the title of the item.
    """
    table(FolderListing)
    grok.context(Interface)
    grok.name('folderlisting.title')

    weight = 10
    header = _(u"Title")

    def renderCell(self, item):
        if isinstance(item, PersistentBroken):
            broken = _(
                "Broken item: ${name}",
                mapping={"name": item.__Broken_state__.get('__name__')})
            return translate(broken)

        return LinkColumn.renderCell(self, item)


class ModificationDate(ModifiedColumn):
    """Displays the last modification date.
    """
    table(FolderListing)
    grok.context(Interface)
    grok.name('folderlisting.modified')

    weight = 20
    header = _(u"Modification date")



# -*- coding: utf-8 -*-

import grok
import dolmen.content

from dolmen.app import security, layout
from zope.component import getUtilitiesFor
from zope.container.constraints import checkFactory
from zope.container.interfaces import IContainer
from zope.security.checker import CheckerPublic
from zope.security.management import checkPermission

grok.templatedir("templates")


class AddMenu(grok.Viewlet):
    grok.order(60)
    grok.view(layout.IDisplayView)
    grok.context(IContainer)
    grok.require(security.CanAddContent)
    grok.viewletmanager(layout.AboveBody)

    def checkFactory(self, name, factory):
        """Verifies the factory and the right of the logged user against
        the container and the required permission of the content type.
        """
        if not checkFactory(self.context, name, factory):
            return False

        permission = dolmen.content.require.bind().get(factory.factory)
        if permission == 'zope.Public':
            permission = CheckerPublic
        return checkPermission(permission, self.context)

    def update(self):
        """Gathers the factories allowed for the context container
        in a list of factories information useable by the template.
        """
        self.factories = []
        self.contexturl = self.view.url(self.context)

        for name, factory in getUtilitiesFor(dolmen.content.IFactory):
            # We iterate and check the factories
            if self.checkFactory(name, factory):
                factory_class = factory.factory
                self.factories.append(dict(
                    name=name,
                    id=name.replace(".", "-"),
                    url='%s/++add++%s' % (self.contexturl, name),
                    title=factory_class.__content_type__,
                    description=(factory.description or
                                 factory_class.__doc__),
                    ))



from setuptools import setup, find_packages
from os.path import join

name = 'dolmen.app.container'
version = '1.0a1'
readme = open(join("src", "dolmen", "app", "container", "README.txt")).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires=[
    'ZODB3',
    'dolmen.app.layout',
    'dolmen.app.security',
    'dolmen.content',
    'dolmen.menu',
    'grok',
    'megrok.z3ctable>=1.4.0',
    'setuptools',
    'zope.component',
    'zope.container',
    'zope.i18n',
    'zope.i18nmessageid',
    'zope.interface',
    'zope.security',
    ]

tests_require = [
    'zope.testing',
    'zope.securitypolicy',
    'zope.annotation',
    'zope.browserresource',
    'zope.configuration',
    'zope.publisher',
    'zope.site',
    'zope.traversing',
    'zope.i18n',
    'zope.principalregistry',
    ]

setup(name = name,
      version = version,
      description = 'Containers tools for Dolmen applications',
      long_description = readme + '\n\n' + history,
      keywords = 'Grok Zope3 CMS Dolmen',
      author = 'Souheil Chelfouh',
      author_email = 'trollfot@gmail.com',
      url = 'http://gitweb.dolmen-project.org/',
      download_url = '',
      license = 'GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages = ['dolmen', 'dolmen.app'],
      include_package_data = True,
      platforms = 'Any',
      zip_safe = False,
      tests_require = tests_require,
      install_requires = install_requires,
      extras_require = {'test': tests_require},
      test_suite="dolmen.app.container",
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
      ],
)
